%{
#include <stdio.h>
#include <string.h>
#define GLOBAL_DECLARATIONS
#include "vhdl_def.h"
#include "err_msg.h"
%}

%token
tok_ACCESS
tok_AFTER
tok_ALIAS
tok_ALL
tok_AND
tok_ARCHITECTURE
tok_ARRAY
tok_ASSERT
tok_ATTRIBUTE
tok_BEGIN
tok_BLOCK
tok_BODY
tok_BUFFER
tok_BUS
tok_CASE
tok_COMPONENT
tok_CONFIGURATION
tok_CONSTANT
tok_DISCONNECT
tok_DOWNTO
tok_ELSE
tok_ELSIF
tok_END
tok_ENTITY
tok_EXIT
tok_FILE
tok_FOR
tok_FUNCTION
tok_GENERATE
tok_GENERIC
tok_GROUP
tok_GUARDED
tok_IF
tok_IMPURE
tok_IN
tok_INERTIAL
tok_INOUT
tok_IS
tok_LABEL
tok_LIBRARY
tok_LINKAGE
tok_LITERAL
tok_LOOP
tok_MAP
tok_NAND
tok_NEW
tok_NEXT
tok_NOR
tok_NULL
tok_OF
tok_ON
tok_OPEN
tok_OR
tok_OTHERS
tok_OUT
tok_PACKAGE
tok_PORT
tok_POSTPONED
tok_PROCEDURE
tok_PROCESS
tok_PURE
tok_RANGE
tok_RECORD
tok_REGISTER
tok_REJECT
tok_REPORT
tok_RETURN
tok_ROL
tok_ROR
tok_SELECT
tok_SEVERITY
tok_SHARED
tok_SIGNAL
tok_SLA
tok_SLL
tok_SRA
tok_SRL
tok_SUBTYPE
tok_THEN
tok_TO
tok_TRANSPORT
tok_TYPE
tok_UNAFFECTED
tok_UNITS
tok_UNTIL
tok_USE
tok_VARIABLE
tok_WAIT
tok_WHEN
tok_WHILE
tok_WITH
tok_XNOR
tok_XOR

/* VHDL binary operators
  ======================= */

%nonassoc tok_EQSym tok_NESym tok_LTSym tok_LESym tok_GTSym tok_GESym
%left tok_Plus tok_Minus tok_Ampersand
%left MED_PRECEDENCE
%left tok_Star tok_Slash tok_MOD tok_REM
%nonassoc tok_DoubleStar tok_ABS tok_NOT MAX_PRECEDENCE

%token tok_Apostrophe
tok_LeftParen
tok_RightParen
tok_Comma
tok_VarAsgn
tok_Colon
tok_Semicolon
tok_Arrow
tok_Box
tok_Bar
tok_Dot

%token tok_Identifier
tok_AbstractLit
tok_CharacterLit
tok_StringLit
tok_BitStringLit


%%
start: 
	| design_file
;
design_file     : design_unit_list
;
design_unit_list : design_unit
		 | design_unit_list design_unit  
;
designator : tok_Identifier	
	   | tok_StringLit  
;
literal    : tok_AbstractLit  
	   | tok_CharacterLit  
	   | tok_BitStringLit
	   | physical_literal_no_default  
	   | tok_NULL  
;
enumeration_literal  : tok_CharacterLit
		     | tok_Identifier  
;
physical_literal     : physical_literal_1 tok_Identifier  
;
physical_literal_1   :  
		     | tok_AbstractLit  
;
physical_literal_no_default    : tok_AbstractLit tok_Identifier  
;
idf_list : tok_Identifier 
	 | idf_list tok_Comma tok_Identifier 
;

/*------------------------------------------
--  Desing Unit
--------------------------------------------*/

design_unit     : context_list lib_unit  
;
context_list    :  
		| context_list context_item  
;
lib_unit    : entity_decl  
	    | config_decl  
	    | package_decl  
	    | arch_body  
	    | package_body  
;
context_item    : lib_clause  
		| use_clause  
;
lib_clause  : tok_LIBRARY idf_list tok_Semicolon  
;
use_clause : tok_USE sel_list tok_Semicolon  
;
sel_list : sel_name		
	 | sel_list tok_Comma sel_name	
;
/*------------------------------------------
--  Library Units
--------------------------------------------*/

entity_decl : tok_ENTITY tok_Identifier tok_IS	entity_decl_1 
	entity_decl_3 entity_decl_4 tok_END entity_decl_5 tok_Semicolon 
;
entity_decl_5 :
	      | tok_ENTITY
	      | tok_Identifier
	      | tok_ENTITY tok_Identifier
;
entity_decl_4 :  
	      | tok_BEGIN concurrent_stats  
;
entity_decl_3 :  
	      | entity_decl_3 entity_decl_6  
;
entity_decl_6 : entity_decltve_item  
;
entity_decl_1 :  
	      | tok_GENERIC interf_list tok_Semicolon  tok_PORT interf_list tok_Semicolon  
	      | tok_GENERIC interf_list tok_Semicolon  
	      | tok_PORT interf_list tok_Semicolon  
;
arch_body : arch_start arch_body_1 tok_BEGIN concurrent_stats tok_END
	arch_body_2 tok_Semicolon 
;
arch_start : tok_ARCHITECTURE tok_Identifier tok_OF tok_Identifier tok_IS 
;
arch_body_2  :  
	     | tok_Identifier  
	     | tok_ARCHITECTURE  
	     | tok_ARCHITECTURE tok_Identifier  
;
arch_body_1  :  
	     | arch_body_1 arch_body_3  
;
arch_body_3  : block_decltve_item  
;
config_decl : config_start config_decl_1 block_config tok_END config_decl_2 tok_Semicolon
;
config_start : tok_CONFIGURATION tok_Identifier tok_OF tok_Identifier tok_IS 
;
config_decl_2    :  
		 | tok_CONFIGURATION tok_Identifier  
		 | tok_CONFIGURATION 
		 | tok_Identifier  
;
config_decl_1    :  
		 | config_decl_1 config_decl_3  
;
config_decl_3    : config_decltve_item  
;
package_decl  : package_start package_decl_1 tok_END package_decl_2 tok_Semicolon  
;
package_start : tok_PACKAGE tok_Identifier tok_IS 
;
package_decl_2     :  
		   | tok_Identifier  
		   | tok_PACKAGE  
		   | tok_PACKAGE tok_Identifier
;
package_decl_1     :  
		   | package_decl_1 package_decl_3  
;
package_decl_3     : package_decltve_item  
;
package_body    : pack_body_start package_body_1 tok_END package_body_2 tok_Semicolon  
;
pack_body_start : tok_PACKAGE tok_BODY tok_Identifier tok_IS 
;
package_body_2  :  
		| tok_Identifier  
		   | tok_PACKAGE tok_BODY
		   | tok_PACKAGE tok_BODY tok_Identifier
;
package_body_1  :  
		| package_body_1 package_body_3  
;
package_body_3  : package_body_decltve_item  
;
/*------------------------------------------
--  Declarative Item
--------------------------------------------*/
common_decltve_item   : type_decl  
		      | subtype_decl  
		      | constant_decl  
		      | alias_decl  
		      | subprog_decl  
		      | use_clause  
		      | file_decl
;
entity_decltve_item   : common_decltve_item  
		      | subprog_body  
		      | attribute_decl  
		      | attribute_spec  
		      | signal_decl  
;
block_decltve_item    : common_decltve_item  
		      | subprog_body  
		      | comp_decl  
		      | attribute_decl  
		      | attribute_spec  
		      | signal_decl  
	              | shared_variable_decl  
;
package_decltve_item  : common_decltve_item  
		      | comp_decl  
		      | attribute_decl  
		      | attribute_spec  
		      | signal_decl  
;
package_body_decltve_item  : common_decltve_item  
			   | subprog_body  
;
subprog_decltve_item    : common_decltve_item  
			| subprog_body  
			| attribute_decl  
			| attribute_spec  
			| variable_decl  
			| shared_variable_decl  
;
procs_decltve_item  : common_decltve_item  
		    | subprog_body  
		    | attribute_decl  
		    | attribute_spec  
		    | variable_decl  
		    | shared_variable_decl  
		    | signal_decl  
;
config_decltve_item : attribute_spec  
		    | use_clause  
;
/*------------------------------------------
--  Subprograms
--------------------------------------------*/

subprog_decl    : subprog_spec tok_Semicolon  
;
subprog_spec  : tok_PROCEDURE tok_Identifier subprog_spec_1  
	      | tok_FUNCTION designator subprog_spec_2 tok_RETURN mark  
;
subprog_spec_2     :  
		   | interf_list  
;
subprog_spec_1     :  
		   | interf_list  
;
subprog_body : subprog_spec tok_IS subprog_body_1 tok_BEGIN seq_stats tok_END subprog_body_2 tok_Semicolon  
;
subprog_body_2    :  
		  | designator  
		  | tok_PROCEDURE
		  | tok_PROCEDURE designator  
		  | tok_FUNCTION
		  | tok_FUNCTION designator  
;
subprog_body_1    :  
		  | subprog_body_1 subprog_body_3  
;
subprog_body_3    : subprog_decltve_item  
;
/*--------------------------------------------------
--  Interface Lists and Associaton Lists
----------------------------------------------------*/

interf_list  : tok_LeftParen interf_element interf_list_1 tok_RightParen  
;
interf_list_1     :  
		  | interf_list_1 interf_list_2  
;
interf_list_2     : tok_Semicolon interf_element  
;
interf_element    : interf_element_1 idf_list tok_Colon interf_element_2 subtype_indic interf_element_3 interf_element_4  
;
interf_element_4  :  
		  | tok_VarAsgn expr  
;
interf_element_3  :  
		  | tok_BUFFER
;
interf_element_2  :  
		  | mode  
;
interf_element_1  :  
		  | object_class  
;
mode  : tok_IN  
      | tok_OUT 
      | tok_INOUT  
      | tok_BUFFER  
      | tok_LINKAGE  
;
association_list     : tok_LeftParen association_element association_list_1 tok_RightParen  
;
association_list_1   :  
		     | tok_Comma association_element association_list_1 
;
gen_association_list : tok_LeftParen gen_association_element gen_association_list_1 tok_RightParen  
;
gen_association_list_1    :  
			  | gen_association_list_1 gen_association_list_2  
;
gen_association_list_2    : tok_Comma gen_association_element  
;
association_element  : actual_part  
		     | formal_part tok_Arrow actual_part
;
gen_association_element   : expr
			  | discrete_range1  
;
formal_part     : name  
;
actual_part     : expr  
		| tok_OPEN  
;
/*--------------------------------------------------
--  Names and Expressions
----------------------------------------------------*/

mark  : tok_Identifier	
      | sel_name  
;
expr  : and_relation  
      | or_relation  
      | rol_relation  
      | ror_relation  
      | sla_relation  
      | sll_relation  
      | sra_relation  
      | srl_relation  
      | xor_relation  
      | xnor_relation
      | relation  
      | relation tok_NAND relation  
      | relation tok_NOR relation  
;
and_relation    : relation tok_AND relation  
		| and_relation tok_AND relation  
;
or_relation     : relation tok_OR relation  
		| or_relation tok_OR relation  
;
rol_relation    : relation tok_ROL relation  
		| rol_relation tok_ROL relation  
;
ror_relation    : relation tok_ROR relation  
		| ror_relation tok_ROR relation  
;
sla_relation    : relation tok_SLA relation  
		| sla_relation tok_SLA relation  
;
sll_relation    : relation tok_SLL relation  
		| sll_relation tok_SLL relation  
;
sra_relation    : relation tok_SRA relation  
		| sra_relation tok_SRA relation  
;
srl_relation    : relation tok_SRL relation  
		| srl_relation tok_SRL relation  
;
xor_relation    : relation tok_XOR relation  
		| xor_relation tok_XOR relation  
;
xnor_relation   : relation tok_XNOR relation  
		| xnor_relation tok_XNOR relation  
;
relation : primary  
	 | tok_Plus primary  %prec MED_PRECEDENCE
	 | tok_Minus primary %prec MED_PRECEDENCE
	 | tok_ABS primary   
	 | tok_NOT primary   
	 | primary tok_DoubleStar primary
	 | relation tok_MOD relation
	 | relation tok_REM relation
	 | relation tok_Ampersand relation
	 | relation tok_Star relation
	 | relation tok_Plus relation
	 | relation tok_Minus relation
	 | relation tok_LESym relation
	 | relation tok_GESym relation
	 | relation tok_LTSym relation
	 | relation tok_GTSym relation
	 | relation tok_EQSym relation
	 | relation tok_NESym relation
	 | relation tok_Slash relation
;

primary    : name  
	   | literal  
	   | aggregate  
	   | qualified_expr  
	   | tok_LeftParen expr tok_RightParen  
	   | function_call
;

name  : mark  
      | name2  
;
name2   : tok_StringLit
	| attribute_name  
	| ifts_name  
;
sel_name   : name tok_Dot suffix  
;
suffix  : designator  
	| tok_CharacterLit
	| tok_ALL  
;
ifts_name  : mark gen_association_list  
	   | name2 gen_association_list  
;
attribute_name  : mark tok_Apostrophe tok_Identifier  
		| name2 tok_Apostrophe tok_Identifier  
		| mark tok_Apostrophe tok_RANGE
		| name2 tok_Apostrophe tok_RANGE
;
aggregate  : element_association_list2 tok_RightParen  
	   | tok_LeftParen choices tok_Arrow expr tok_RightParen  
;
element_association_list2 : tok_LeftParen element_association tok_Comma element_association  
			  | element_association_list2 tok_Comma element_association  
;
qualified_expr  : mark tok_Apostrophe tok_LeftParen expr tok_RightParen  
		| mark tok_Apostrophe aggregate  
;

function_call : mark association_list
	      | tok_StringLit association_list
;
/*--------------------------------------------------
--  Element Association and Choices
----------------------------------------------------*/

element_association  : choices tok_Arrow expr  
		     | expr  
;
choices    : choice choices_1  
;
choices_1  :  
	   | choices_1 choices_2  
;
choices_2  : tok_Bar choice  
;
choice  : expr  
	| discrete_range1  
	| tok_OTHERS  
;
/*--------------------------------------------------
--	Type Declarations
----------------------------------------------------*/

type_decl     : tok_TYPE tok_Identifier tok_IS type_definition tok_Semicolon 
;
type_definition : enumeration_type_definition  
		| range_constraint  
		| physical_type_definition  
		| unconstrained_array_definition  
		| constrained_array_definition  
		| record_type_definition  
		| access_type_definition  
		| file_type_definition  
;
enumeration_type_definition    : tok_LeftParen enumeration_literal enumeration_type_definition_1 tok_RightParen  
;
enumeration_type_definition_1  :  
			       | enumeration_type_definition_1 enumeration_type_definition_2  
;
enumeration_type_definition_2  : tok_Comma enumeration_literal  
;
physical_type_definition : range_constraint tok_UNITS base_unit_decl
	physical_type_definition_1 tok_END tok_UNITS 
;
physical_type_definition_1 :  
			   | physical_type_definition_1 physical_type_definition_2  
;
physical_type_definition_2 : secondary_unit_decl  
;
base_unit_decl     : tok_Identifier tok_Semicolon  
;
secondary_unit_decl     : tok_Identifier tok_EQSym physical_literal tok_Semicolon  
;
unconstrained_array_definition : tok_ARRAY tok_LeftParen
	index_subtype_definition unconstrained_array_definition_1 tok_RightParen tok_OF
	subtype_indic 
;
unconstrained_array_definition_1 :  
				 | unconstrained_array_definition_1 unconstrained_array_definition_2  
;
unconstrained_array_definition_2 : tok_Comma index_subtype_definition  
;
index_subtype_definition  : mark tok_RANGE tok_Box  
;
constrained_array_definition   : tok_ARRAY index_constraint tok_OF subtype_indic  
;
record_type_definition : tok_RECORD element_decl record_type_definition_1 tok_END
	tok_RECORD record_type_definition_3
;
record_type_definition_1 :  
			 | record_type_definition_1 record_type_definition_2  
;
record_type_definition_2  : element_decl  
;
record_type_definition_3 :
                         | tok_Identifier
;
element_decl  : idf_list tok_Colon subtype_indic tok_Semicolon  
;
access_type_definition    : tok_ACCESS subtype_indic  
;
file_type_definition : tok_FILE tok_OF mark  
;
/*--------------------------------------------------
--  Subtypes and Constraints
----------------------------------------------------*/

subtype_decl  : tok_SUBTYPE tok_Identifier tok_IS subtype_indic tok_Semicolon  
;
subtype_indic   : mark subtype_indic_1  
		| subtype_indic1  
;
subtype_indic_1 :  
		| gen_association_list  
;
subtype_indic1  : mark mark range_constraint  
		| mark range_constraint  
		| mark mark subtype_indic1_1  
;
subtype_indic1_1 :  
		 | gen_association_list  
;
range_constraint     : tok_RANGE range_spec  
;
index_constraint     : tok_LeftParen discrete_range index_constraint_1 tok_RightParen  
;
index_constraint_1  :  
		    | index_constraint_1 index_constraint_2  
;
index_constraint_2   : tok_Comma discrete_range  
;
discrete_range  : subtype_indic  
		| range_spec  
;
discrete_range1 : subtype_indic1  
		| expr direction expr  
;
range_spec : attribute_name  
	   | expr direction expr  
;
direction : tok_TO
	  | tok_DOWNTO  
;
/*--------------------------------------------------
--  Objects, Aliases
----------------------------------------------------*/

constant_decl : tok_CONSTANT idf_list tok_Colon subtype_indic constant_decl_1 tok_Semicolon  
;
constant_decl_1 :  
		| tok_VarAsgn expr  
;
signal_decl   : tok_SIGNAL idf_list tok_Colon subtype_indic tok_Semicolon  
	      | tok_SIGNAL idf_list tok_Colon subtype_indic tok_VarAsgn expr tok_Semicolon  
;
variable_decl : tok_VARIABLE idf_list tok_Colon subtype_indic tok_Semicolon  
	      | tok_VARIABLE idf_list tok_Colon subtype_indic tok_VarAsgn expr tok_Semicolon  
;
shared_variable_decl : tok_SHARED variable_decl
;
object_class    : tok_CONSTANT  
		| tok_SIGNAL  
		| tok_VARIABLE  
;
alias_decl    : tok_ALIAS tok_Identifier tok_Colon subtype_indic tok_IS name tok_Semicolon  
;
file_decl     : tok_FILE tok_Identifier tok_Colon subtype_indic tok_IS file_mode name tok_Semicolon  
;
file_mode     : 
	      | tok_IN
	      | tok_OUT
;
/*--------------------------------------------------
--  Attribute Declarations and Specifications
----------------------------------------------------*/

attribute_decl     : tok_ATTRIBUTE tok_Identifier tok_Colon mark tok_Semicolon  
;
/* TODO
 * Zastapic tok_Identifier produkcja dopuszczalnych atrybutow
 */
attribute_spec   : tok_ATTRIBUTE tok_Identifier tok_OF entity_spec tok_IS expr tok_Semicolon  
;
entity_spec : entity_name_list tok_Colon entity_class  
;
entity_name_list : designator entity_name_list_1  
		 | tok_OTHERS  
		 | tok_ALL  
;
entity_name_list_1 :  
		   | entity_name_list_1 entity_name_list_2  
;
entity_name_list_2   : tok_Comma designator  
;
entity_class    : tok_ENTITY
		| tok_ARCHITECTURE  
		| tok_PACKAGE 
		| tok_COMPONENT
		| tok_LABEL  
		| tok_TYPE
		| tok_SUBTYPE  
		| tok_PROCEDURE  
		| tok_FUNCTION  
		| tok_SIGNAL
		| tok_VARIABLE
		| tok_CONSTANT  
;
/*--------------------------------------------------
--  Schemes
----------------------------------------------------*/

generation_scheme  : if_scheme  
		   | for_scheme  
;
iteration_scheme   : for_scheme  
		   | while_scheme  
;
if_scheme  : tok_IF expr  
;
for_scheme : tok_FOR tok_Identifier tok_IN discrete_range  
;
while_scheme    : tok_WHILE expr  
;
/*--------------------------------------------------
--  Concurrent Statements
----------------------------------------------------*/

concurrent_stats   : concurrent_stats_1  
;
concurrent_stats_1 :  
		   | concurrent_stats_1 concurrent_stats_2  
;
concurrent_stats_2 : concurrent_stat  
;
concurrent_stat : block_stat  
	| concurrent_assertion_stat  
	| concurrent_signal_assign_stat  
	| concurrent_procedure_call 
	| comp_inst_stat
	| generate_stat  
	| procs_stat  
;
block_stat : tok_Identifier tok_Colon tok_BLOCK block_stat_1 block_stat_9 block_stat_2
	block_stat_3 block_stat_4 tok_BEGIN concurrent_stats tok_END tok_BLOCK 
	block_stat_5 tok_Semicolon 
;
block_stat_5    :  
		| tok_Identifier  
;
block_stat_4    :  
		| block_stat_4 block_stat_6  
;
block_stat_6    : block_decltve_item  
;
block_stat_3    :  
		| tok_PORT interf_list tok_Semicolon block_stat_7  
;
block_stat_7    :  
		| tok_PORT tok_MAP association_list tok_Semicolon  
;
block_stat_2    :  
		| tok_GENERIC interf_list tok_Semicolon block_stat_8  
;
block_stat_8    :  
		| tok_GENERIC tok_MAP association_list tok_Semicolon  
;
block_stat_1    :  
		| tok_LeftParen expr tok_RightParen  
;
block_stat_9	:
		| tok_IS
;

comp_inst_stat  : tok_Identifier tok_Colon comp_inst_stat_2 
;
comp_inst_stat_2 : mark comp_inst_stat_5 tok_Semicolon  
		 | tok_ENTITY mark comp_inst_stat_5 tok_Semicolon  
		 | tok_COMPONENT mark comp_inst_stat_5 tok_Semicolon  
;
comp_inst_stat_3 : 
		 | tok_GENERIC tok_MAP association_list 
;
comp_inst_stat_4 : 
		 | tok_PORT tok_MAP association_list 
;
comp_inst_stat_5 : comp_inst_stat_3  comp_inst_stat_4 
;
concurrent_assertion_stat : tok_Identifier tok_Colon assertion_stat  
			  | assertion_stat  
;
concurrent_procedure_call : tok_Identifier tok_Colon function_call tok_Semicolon
			  |function_call tok_Semicolon
;
concurrent_signal_assign_stat   : tok_Identifier tok_Colon condal_signal_assign  
				| condal_signal_assign  
				| tok_Identifier tok_Colon sel_signal_assign  
				| sel_signal_assign  
;
condal_signal_assign  : target tok_LESym opts condal_wavefrms tok_Semicolon  
		      | target tok_Semicolon 
;
condal_wavefrms   : condal_wavefrms_1 wavefrm  
;
condal_wavefrms_1 :  
		  | condal_wavefrms_1 condal_wavefrms_2  
;
condal_wavefrms_2 : wavefrm tok_WHEN expr tok_ELSE  
;
wavefrm   : wavefrm_element wavefrm_1  
;
wavefrm_1 :  
	  | wavefrm_1 wavefrm_2  
;
wavefrm_2 : tok_Comma expr
;
wavefrm_element : expr 
                | expr tok_AFTER expr
;
target  : name  
	| aggregate  
;
opts    :  
	| tok_TRANSPORT  
	| tok_INERTIAL  
	| tok_GUARDED
;
sel_signal_assign : tok_WITH expr tok_SELECT target tok_LESym opts sel_wavefrms tok_Semicolon 
;
sel_wavefrms    : sel_wavefrms_1 wavefrm tok_WHEN choices  
;
sel_wavefrms_1  :  
		| sel_wavefrms_1 sel_wavefrms_2  
;
sel_wavefrms_2  : wavefrm tok_WHEN choices tok_Comma  
;
generate_stat : tok_Identifier tok_Colon generation_scheme tok_GENERATE generate_stat_2
	concurrent_stats tok_END tok_GENERATE generate_stat_1 tok_Semicolon 
;
generate_stat_1 :  
		| tok_Identifier  
;
generate_stat_2 :
		| tok_BEGIN
		| block_decltve_item tok_BEGIN 
;
procs_stat      : tok_Identifier tok_Colon procs_stat1  
		| procs_stat1  
;
procs_stat1 : tok_PROCESS procs_stat1_1 procs_stat1_2 tok_BEGIN seq_stats tok_END
	tok_PROCESS procs_stat1_3 tok_Semicolon 
;
procs_stat1_3   :  
		| tok_Identifier  
;
procs_stat1_2   :  
		| procs_stat1_2 procs_stat1_4  
;
procs_stat1_4   : procs_decltve_item  
;
procs_stat1_1   :  
		| tok_IS
		| tok_LeftParen sensitivity_list tok_RightParen 
		| tok_LeftParen sensitivity_list tok_RightParen tok_IS
;
sensitivity_list     : name sensitivity_list_1  
;
sensitivity_list_1   :  
		     | sensitivity_list_1 sensitivity_list_2  
;
sensitivity_list_2   : tok_Comma name  
;
/*--------------------------------------------------
--  Sequential Statements
----------------------------------------------------*/

seq_stats     : seq_stats_1  
;
seq_stats_1   :  
	      | seq_stats_1 seq_stats_2  
;
seq_stats_2   : seq_stat  
;
seq_stat : assertion_stat  
	 | case_stat  
	 | exit_stat  
	 | if_stat  
	 | loop_stat  
	 | next_stat  
	 | null_stat
	 | return_stat  
	 | wait_stat  
	 | seq_simple_stats
	 | aggregate_seq_stat
;
assertion_stat  : tok_Identifier tok_Colon tok_ASSERT expr assertion_stat_1 assertion_stat_2 tok_Semicolon  
                | tok_ASSERT expr assertion_stat_1 assertion_stat_2 tok_Semicolon  
;
assertion_stat_2 :  
		 | tok_SEVERITY expr  
;
assertion_stat_1 :  
		 | tok_REPORT expr  
;
case_stat  : tok_Identifier tok_Colon tok_CASE expr tok_IS case_stat_alternative case_stat_1 tok_END tok_CASE case_stat_3 tok_Semicolon  
           | tok_CASE expr tok_IS case_stat_alternative case_stat_1 tok_END tok_CASE case_stat_3 tok_Semicolon  
;
case_stat_1     :  
		| case_stat_1 case_stat_2  
;
case_stat_2     : case_stat_alternative  
;
case_stat_3 	:
		| tok_Identifier
;
case_stat_alternative     : tok_Identifier tok_Colon tok_WHEN choices tok_Arrow seq_stats  
                          | tok_WHEN choices tok_Arrow seq_stats  
;
exit_stat  : tok_Identifier tok_Colon tok_EXIT exit_stat_1 exit_stat_2 tok_Semicolon  
           | tok_EXIT exit_stat_1 exit_stat_2 tok_Semicolon  
;
exit_stat_2     :  
		| tok_WHEN expr  
;
exit_stat_1     :  
		| tok_Identifier  
;
if_stat    : tok_Identifier tok_Colon tok_IF expr tok_THEN seq_stats if_stat_1 if_stat_2 tok_END tok_IF if_stat_4 tok_Semicolon  
           | tok_IF expr tok_THEN seq_stats if_stat_1 if_stat_2 tok_END tok_IF if_stat_4 tok_Semicolon  
;
if_stat_2  :  
	   | tok_ELSE seq_stats  
;
if_stat_1  :  
	   | if_stat_1 if_stat_3  
;
if_stat_3  : tok_ELSIF expr tok_THEN seq_stats  
;
if_stat_4     :  
	      | tok_Identifier  
;

loop_stat  : tok_Identifier tok_Colon loop_stat_2 tok_LOOP seq_stats tok_END tok_LOOP loop_stat_3 tok_Semicolon  
           | loop_stat_2 tok_LOOP seq_stats tok_END tok_LOOP loop_stat_3 tok_Semicolon  
;
loop_stat_3     :  
		| tok_Identifier  
;
loop_stat_2     :  
		| iteration_scheme  
;

next_stat  : tok_Identifier tok_Colon tok_NEXT next_stat_1 next_stat_2 tok_Semicolon  
           | tok_NEXT next_stat_1 next_stat_2 tok_Semicolon  
;
next_stat_2     :  
		| tok_WHEN expr  
;
next_stat_1     :  
		| tok_Identifier  
;
null_stat  : tok_NULL tok_Semicolon  
;
return_stat     : tok_Identifier tok_Colon tok_RETURN return_stat_1 tok_Semicolon  
                | tok_RETURN return_stat_1 tok_Semicolon  
;
return_stat_1   :  
		| expr  
;
seq_simple_stats      : tok_Identifier tok_Colon simple_left_stat seq_simple_stats_1 tok_Semicolon 
		      | simple_left_stat seq_simple_stats_1 tok_Semicolon 
;
simple_left_stat : /* simple_left_stat_1
		 | simple_left_stat tok_Dot simple_left_stat_1 */
		 | name
		 | aggregate
;
simple_left_stat_1 : tok_Identifier simple_left_stat_2 
		   | tok_Identifier tok_LeftParen simple_left_choices tok_RightParen
;
simple_left_stat_2 : 
		   | tok_LeftParen expr tok_RightParen
		   | simple_left_stat_2 tok_LeftParen expr tok_RightParen
;
simple_left_choices : simple_left_choice
		    | simple_left_choices tok_Comma simple_left_choice 
;
simple_left_choice : expr
		   | tok_Identifier tok_Arrow expr
		   | tok_AbstractLit tok_Arrow expr
		   | discrete_range1
;
seq_simple_stats_1 :
		   | tok_LESym signal_assign_stat_1 wavefrm 
		   | tok_VarAsgn expr 
;
/*
signal_assign_stat    : tok_Identifier tok_Colon target tok_LESym signal_assign_stat_1 wavefrm tok_Semicolon  
                      | target tok_LESym signal_assign_stat_1 wavefrm tok_Semicolon  
;
*/
signal_assign_stat_1  :  
		      | tok_TRANSPORT  
;
/*
variable_assign_stat  : tok_Identifier tok_Colon target tok_VarAsgn expr tok_Semicolon  
                      | target tok_VarAsgn expr tok_Semicolon  
;
*/
aggregate_seq_stat : tok_Identifier tok_Colon aggregate aggregate_seq_stat_1 tok_Semicolon
		   | aggregate aggregate_seq_stat_1 tok_Semicolon
;
aggregate_seq_stat_1 : tok_VarAsgn expr 
		     | tok_LESym expr 
wait_stat  : tok_Identifier tok_Colon tok_WAIT wait_stat_1 wait_stat_2 wait_stat_3 tok_Semicolon  
           | tok_WAIT wait_stat_1 wait_stat_2 wait_stat_3 tok_Semicolon  
;
wait_stat_3     :  
		| tok_FOR expr  
;
wait_stat_2     :  
		| tok_UNTIL expr  
;
wait_stat_1     :  
		| tok_ON sensitivity_list  
;
/*--------------------------------------------------
--  Components and Configurations
----------------------------------------------------*/

comp_decl     : tok_COMPONENT tok_Identifier comp_decl_1 tok_END tok_COMPONENT comp_decl_3 tok_Semicolon  
;
comp_decl_1   :  
	      | tok_IS
	      | tok_IS comp_decl_2  
	      | comp_decl_2  
;
comp_decl_2   : tok_GENERIC interf_list tok_Semicolon  tok_PORT interf_list tok_Semicolon  
	      | tok_PORT interf_list tok_Semicolon  
;
comp_decl_3   :
	      | tok_Identifier
;
/*
block_config  : tok_FOR block_spec block_config_1 block_config_2 tok_END tok_FOR tok_Semicolon  
;
block_config_2  :  
		| block_config_2 block_config_3  
;
block_config_3  : config_item  
;
block_config_1  :  
		| block_config_1 block_config_4 block_config_5 
;
block_config_4     : tok_USE sel_list 
		   | tok_USE tok_CONFIGURATION sel_list 
		   | tok_USE tok_ENTITY sel_list 
;
block_config_5	: comp_inst_stat_5 
;
block_spec  : name  
	    | tok_Identifier tok_Colon name
	    | tok_ALL tok_Colon name
;
*/
block_config : tok_FOR tok_Identifier block_config_2 tok_END tok_FOR tok_Semicolon
;
block_config_2 :
               | block_config_2 block_config_1
               | block_config_1
;
block_config_1 : tok_FOR comp_spec bind_indic tok_Semicolon tok_END tok_FOR tok_Semicolon
               | tok_FOR comp_spec bind_indic tok_Semicolon block_config tok_END tok_FOR tok_Semicolon
;
comp_spec : tok_OTHERS tok_Colon tok_Identifier
          | tok_ALL tok_Colon tok_Identifier
          | tok_Identifier tok_Colon tok_Identifier
;
bind_indic : tok_USE bind_indic_1 bind_indic_2 bind_indic_3
;
bind_indic_1 : tok_ENTITY name
             | tok_ENTITY name tok_LeftParen tok_Identifier tok_RightParen
             | tok_CONFIGURATION name
;
bind_indic_2 :
             | tok_GENERIC tok_MAP association_list
;
bind_indic_3 :
             | tok_PORT tok_MAP association_list
;
/*
config_item	: block_config
;
*/
%%

int yyerror(char *str)
  {
  char *pos, localmsg[1024];

  /* Replace trailing newline with ascii zero */
  pos=strchr(current_token,'\n');
  if(pos)
    *pos='\0';

  /* Look for the beginning of a token */
  pos=current_token;
  while( *pos==' ' || *pos=='\t' || *pos==':')
    pos++;

  /* Prepaire a message */
  sprintf(localmsg,"Unexpected token: <%s>", pos);

  ErrorMsg( 5002, localmsg);
  return 10;
  }
