#include <stdio.h>
#include <string.h>
#include <ctype.h>
#if 0
#include <search.h>
#endif
#include "vhdl_def.h"
#if 0
#include "vhdl.tab.h"
#else
#include "vhdl_tab.h"
#endif
#include "err_msg.h"
#ifndef tok_YET_UNKNOWN
#define tok_YET_UNKNOWN -1
#endif


int ParsePragmaLines(char *);
#define PRAGMA_LINE 3
#define PRAGMA_ALDEC_T_OFF 2
#define PRAGMA_SYNOPSYS 1
#define PRAGMA_DEFAULT 0

static int tokentable[]=
  {
    tok_YET_UNKNOWN,    /* there are some reasons ... */
/*
 * Keyword tokens
 */
    tok_ABS,            /* abs    */
    tok_ACCESS,         /* access */
    tok_AFTER,          /* after  */
    tok_ALIAS,          /* alias  */
    tok_ALL,            /* all    */
    tok_AND,            /* and    */
    tok_ARCHITECTURE,   /* architecture */
    tok_ARRAY,          /* array */
    tok_ASSERT,         /* assert */
    tok_ATTRIBUTE,      /* attribute */

    tok_BEGIN,          /* begin */
    tok_BLOCK,          /* block */
    tok_BODY,           /* body  */
    tok_BUFFER,         /* buffer */
    tok_BUS,            /* bus */

    tok_CASE,           /* case */
    tok_COMPONENT,      /* component */
    tok_CONFIGURATION,  /* configuration */
    tok_CONSTANT,       /* constant */

    tok_DISCONNECT,     /* disconnect */
    tok_DOWNTO,         /* downto */

    tok_ELSE,           /* else */
    tok_ELSIF,          /* elsif */
    tok_END,            /* end */
    tok_ENTITY,         /* entity */
    tok_EXIT,           /* exit */

    tok_FILE,           /* file */
    tok_FOR,            /* for */
    tok_FUNCTION,       /* function */

    tok_GENERATE,       /* generate */
    tok_GENERIC,        /* generic */
    tok_GROUP,          /* group */
    tok_GUARDED,        /* guarded */

    tok_IF,             /* if */
    tok_IMPURE,         /* impure */
    tok_IN,             /* in */
    tok_INERTIAL,       /* inertial */
    tok_INOUT,          /* inout */
    tok_IS,             /* is */

    tok_LABEL,          /* label */
    tok_LIBRARY,        /* library */
    tok_LINKAGE,        /* linkage */
    tok_LITERAL,        /* literal */
    tok_LOOP,           /* loop */

    tok_MAP,            /* map */
    tok_MOD,            /* mod */

    tok_NAND,           /* nand */
    tok_NEW,            /* new */
    tok_NEXT,           /* next */
    tok_NOR,            /* nor */
    tok_NOT,            /* not */
    tok_NULL,           /* null */

    tok_OF,             /* of */
    tok_ON,             /* on */
    tok_OPEN,           /* open */
    tok_OR,             /* or */
    tok_OTHERS,         /* others */
    tok_OUT,            /* out */

    tok_PACKAGE,        /* package */
    tok_PORT,           /* port */
    tok_POSTPONED,      /* postponed */
    tok_PROCEDURE,      /* procedure */
    tok_PROCESS,        /* process */
    tok_PURE,           /* pure */

    tok_RANGE,          /* range */
    tok_RECORD,         /* record */
    tok_REGISTER,       /* register */
    tok_REJECT,         /* reject */
    tok_REM,            /* rem */
    tok_REPORT,         /* report */
    tok_RETURN,         /* return */
    tok_ROL,            /* rol */
    tok_ROR,            /* ror */

    tok_SELECT,         /* select */
    tok_SEVERITY,       /* severity */
    tok_SIGNAL,         /* signal */
    tok_SHARED,         /* shared */
    tok_SLA,            /* sla */
    tok_SLL,            /* sll */
    tok_SRA,            /* sra */
    tok_SRL,            /* srl */
    tok_SUBTYPE,        /* subtype */

    tok_THEN,           /* then */
    tok_TO,             /* to */
    tok_TRANSPORT,      /* transport */
    tok_TYPE,           /* type */

    tok_UNAFFECTED,     /* unaffected */
    tok_UNITS,          /* units */
    tok_UNTIL,          /* until */
    tok_USE,            /* use */

    tok_VARIABLE,       /* variable */

    tok_WAIT,           /* wait */
    tok_WHEN,           /* when */
    tok_WHILE,          /* while */
    tok_WITH,           /* with */

    tok_XNOR,           /* xnor */
    tok_XOR,            /* xor */

    tok_YET_UNKNOWN,    /* reserved (for future use or something) */
    tok_YET_UNKNOWN,    /* reserved (for future use or something) */
    tok_YET_UNKNOWN,    /* reserved (for future use or something) */

/*
 * Special character tokens
 */
    tok_YET_UNKNOWN,    /* " */ 
    tok_YET_UNKNOWN,    /* # */
    tok_Ampersand,      /* & */
    tok_Apostrophe,     /* ' */
    tok_LeftParen,      /* ( */
    tok_RightParen,     /* ) */
    tok_Star,           /* * */
    tok_Plus,           /* + */
    tok_Comma,          /* , */
    tok_Minus,          /* - */
    tok_Dot,            /* . */
    tok_Slash,          /* / */
    tok_Colon,          /* : */
    tok_Semicolon,      /* ; */
    tok_LTSym,          /* < */
    tok_EQSym,          /* = */
    tok_GTSym,          /* > */

    tok_YET_UNKNOWN,    /* _ */

    tok_Bar,            /* | */
    tok_Bar,            /* ! */

    tok_YET_UNKNOWN,    /* $ */
    tok_YET_UNKNOWN,    /* % */
    tok_YET_UNKNOWN,    /* ? */
    tok_YET_UNKNOWN,    /* @ */
    tok_YET_UNKNOWN,    /* [ */
    tok_YET_UNKNOWN,    /* \ */
    tok_YET_UNKNOWN,    /* ] */
    tok_YET_UNKNOWN,    /* ^ */
    tok_YET_UNKNOWN,    /* ` */
    tok_YET_UNKNOWN,    /* { */
    tok_YET_UNKNOWN,    /* } */
    tok_YET_UNKNOWN,    /* ~ */
    tok_YET_UNKNOWN,    /* inverted ! */
    tok_YET_UNKNOWN,    /* cent sign */
    tok_YET_UNKNOWN,    /* pound sign */
    tok_YET_UNKNOWN,    /* currency sign */
    tok_YET_UNKNOWN,    /* yen sign */
    tok_YET_UNKNOWN,    /* broken bar */
    tok_YET_UNKNOWN,    /* paragraph sign */
    tok_YET_UNKNOWN,    /* diaeresis */
    tok_YET_UNKNOWN,    /* copyright sign */
    tok_YET_UNKNOWN,    /* feminine ordinal indicator */
    tok_YET_UNKNOWN,    /* left angle quotation mark */
    tok_YET_UNKNOWN,    /* not sign */
    tok_YET_UNKNOWN,    /* soft hyphen */
    tok_YET_UNKNOWN,    /* registered trade mark sign */
    tok_YET_UNKNOWN,    /* macron */
    tok_YET_UNKNOWN,    /* degree sign */
    tok_YET_UNKNOWN,    /* plus-minus sign */
    tok_YET_UNKNOWN,    /* superscript two */
    tok_YET_UNKNOWN,    /* superscript three */
    tok_YET_UNKNOWN,    /* acute accent */
    tok_YET_UNKNOWN,    /* micro sign */
    tok_YET_UNKNOWN,    /* pilcrow sign */
    tok_YET_UNKNOWN,    /* middle dot */
    tok_YET_UNKNOWN,    /* cedilla */
    tok_YET_UNKNOWN,    /* superscript one */
    tok_YET_UNKNOWN,    /* masculine ordinal indicator */
    tok_YET_UNKNOWN,    /* right angle quotation mark */
    tok_YET_UNKNOWN,    /* vulgar fraction one quarter */
    tok_YET_UNKNOWN,    /* vulgar fraction one half */
    tok_YET_UNKNOWN,    /* vulgar fraction three quarters */
    tok_YET_UNKNOWN,    /* inverted question mark */
    tok_YET_UNKNOWN,    /* multiplication sign */
    tok_YET_UNKNOWN,    /* division sign */

/*
 * Special double character tokens
 */
    tok_Arrow,          /* => */
    tok_DoubleStar,     /* ** */
    tok_VarAsgn,        /* := */
    tok_NESym,          /* /= */
    tok_GESym,          /* >= */
    tok_LESym,          /* <= */
    tok_Box             /* <> */
  };

int yylex()
  {
  static char line[16384];
  char  wrd1[32];
  int token_num,i;
  
  if( !fgets( line, 16383, yyin))
    return -1;
  if( line[0]=='#')
    {
#if 0
    sscanf( line,"%s%d%s", wrd1, &(input_file_info.line),
	    input_file_info.name);
    return yylex();
#else
    if(ParsePragmaLines(line))
      return -1;
#endif
    }
  current_token=strchr(line,':');
  current_token++;
  while(*current_token==' ' || *current_token=='\t')
     current_token++;
  for( i=0; current_token[i]; i++)
    if( current_token[i]=='\n')
      current_token[i]='\0';
  sscanf(line,"%d%s", &token_num, wrd1);
#if 0
  printf("%d\n",token_num);
#endif
  if( token_num == -1)
    {
    ErrorMsg(5003,current_token);
    return yylex();
    }
  if( token_num <= 173)
    return tokentable[token_num];
  if( !stricmp(wrd1,"identifier:")          ||
      !stricmp(wrd1,"extended_identifier:") )
    return tok_Identifier;
  if( !stricmp(wrd1,"character_constant:"))
    return tok_CharacterLit;
  if( !stricmp(wrd1,"string_constant:"))
    return tok_StringLit;
  if( !stricmp(wrd1,"bit_string_constant_bin:") ||
      !stricmp(wrd1,"bit_string_constant_oct:") ||
      !stricmp(wrd1,"bit_string_constant_hex:") )
    return tok_BitStringLit;
  if( !stricmp(wrd1,"integer_constant:")              || 
      !stricmp(wrd1,"integer_based_constant:")        ||
      !stricmp(wrd1,"floating_point_constant:")       ||
      !stricmp(wrd1,"floating_point_based_constant:") )
    return tok_AbstractLit;
  return -1;
  }
    


int ParsePragmaLines(char *line)
  {
  char kindofstr[256],optionstr[256],filestr[256];
  int kindof;
  do
    {
    sscanf(line+1,"%s%s",kindofstr,optionstr);
    if(!stricmp(kindofstr,"line"))
      kindof=PRAGMA_LINE;
    else
      if(!stricmp(kindofstr,"aldec_t_off"))
	kindof=PRAGMA_ALDEC_T_OFF;
      else
	if(!stricmp(kindofstr,"synposys"))
	  kindof=PRAGMA_SYNOPSYS;
	else
	  kindof=PRAGMA_DEFAULT;
    switch(kindof)
      {
      case PRAGMA_LINE: /* #line comments */
	sscanf( line,"%s%d%s", kindofstr, /* useless but required*/
		&(input_file_info.line),  /* important */
		input_file_info.name);    /* important */
		break;
      case PRAGMA_ALDEC_T_OFF: /* comment it all! */
	do
	  {
	  if(!fgets( line, 16383, yyin))
	    return -1;
	  sscanf(line+1,"%s%s",kindofstr,optionstr);
	  stricmp(kindofstr,"aldec_t_off");
	  }
	while(stricmp(kindofstr,"aldec_t_on"));
	break;
      case PRAGMA_SYNOPSYS: /* more complex comment - check syntax */
	break;
      default: /* do not know what do - do nothing */
	break;
      }
    
    if( !fgets( line, 16383, yyin))
      return -1;
    }
  while(line[0]=='#');
  return 0;
  }
